#!/usr/bin/perl

#
# fritzwatch Setup-Programm fr LinVDR
# 01.05.2006 <neves-enin at gmx.net>
# 
# Die meisten Funktionen stammen von dem LinVDR Setup-Programm SETUP.pl
# von <cooper at linvdr.org> und <tom at linvdr.org>
# 
# 27.05.2006 Handling von internen Rufnummern hinzugefgt. Dank an
# Andreas Zemke frs Testen.
#

use strict;
use UI::Dialog::Backend::CDialog;
use POSIX qw(strftime sprintf setlocale);
use Getopt::Std;
use VDRTools::Config;
use VDRTools::Sysconfig;
use VDRTools::File;

my $VERSION       = "0.0.5";

# my $SYSCONFIG     = "/etc/sysconfig";
my $CALL_DB       = "/root/caller.db";
my $CALL_DB_BAK   = "/root/caller.db.bak";
my $TITLE         = "fritzwatch - Setup";

my $SYSCONFIG;
my $CALLER;

my %args;

getopts("v", \%args);

if ($args{'v'}){
	print( $0 . ' ' . $VERSION . "\n" );
	exit (0);
}

my $d = new UI::Dialog::Backend::CDialog ( 
	# XXX autosize dont work with checkbox
	backtitle => $TITLE,
	debug => 0, 
	height => 20, 
	width => 70,
	"ok-label" => "Ok",
	"cancel-label" => "Abbruch");
my($return, @return);

####################################

$SYSCONFIG = VDRTools::Sysconfig->new();
exit 1 unless ( defined( $SYSCONFIG ) );

$CALLER = VDRTools::Config->new( $CALL_DB );

if ( not defined( $CALLER ) and -e $CALL_DB ) {
	if ( not -e $CALL_DB_BAK ) {
		exit 1 unless ( vt_mv( $CALL_DB, $CALL_DB_BAK ) );
	}
	else {
		vt_rm( $CALL_DB );
	}
	
	exit 1 if ( -e $CALL_DB );
	
	$CALLER = VDRTools::Config->new( $CALL_DB );
	
	local *FILE;
	open(FILE, $CALL_DB_BAK);
	while(<FILE>) {
		next if(/^#/);   
		/^(\*{0,2}\d+) (.*)/;
		my $key = $1;
		my $value = $2 ? $2 : "";
		$CALLER->{$key} = $value;
	}
	close(FILE);
}

Menu();

#########################################################

sub Menu {
  my $exit = 0;
  notRootWarning();
  while(!$exit) {
    my $return = $d->menu(
      title => "Hauptmenue",
      text  => "Bitte auswaehlen:",
      menuheight => 9,
      height => 16,
      list  => [
        "Ein/Aus", "fritzwatch benutzen?",
        "Fritz!Box", "Auf welche Fritz!Box lauschen",
        "Neue MSN", "Neue MSN eingeben",
        "MSN", "MSN bearbeiten",
        "Neue Nummer", "Neue Nummer eingeben",
        "Nummern",  "Nummern bearbeiten",
        "Anzeige (Anrufer)", "Anzeigeformat fuer Anrufer einstellen",
        "Anzeige (Angerufener)", "Anzeigeformat fuer Angerufenen einstellen",
        "Beenden",  "Konfiguration verlassen",
      ]
    );
    ($exit = 1) if($d->state ne "OK");
    if($return eq "Ein/Aus") {
      MenuFritzwatchOnOff();
    } 
    if($return eq "Fritz!Box") {
      MenuFritzBox();
    } 
    if($return eq "MSN") {
      MenuMSNEdit();
    }
    if($return eq "Neue MSN") {
      MenuNewMSN();
    }
    if($return eq "Nummern") {
      MenuCallerEdit();
    }
    if($return eq "Neue Nummer") {
      MenuNewCaller();
    }
    if($return eq "Anzeige (Anrufer)") {
      MenuDisplay();
    }
    if($return eq "Anzeige (Angerufener)") {
      MenuDisplaySelf();
    }
    if($return eq "Beenden") {
      $return = $d->yesno(
        title => "fritzwatch-Setup Beenden",
        text  => [ "Konfiguration speichern?" ],
        height=> 6,
      );
      if($return) {
       $SYSCONFIG->save();
       $CALLER->save();
      }
      $exit = 1;
    }
  }
}

sub MenuFritzBox {
	$return = $d->inputbox(
		title => "Fritz!Box bekanntgeben",
		text => ["Eine Aenderung ist eigentlich nur notwendig,\nwenn mehrere Fitz!Boxen im Netzwerk sind."],
		entry => $SYSCONFIG->{"FRITZWATCH_FRITZBOX"},
		height => 11);
	$SYSCONFIG->{"FRITZWATCH_FRITZBOX"} = $return if ( $return );
}

sub MenuDisplaySelf {
  my @list;
  my ( $caller, $iscalled ) = split ( /,/, $SYSCONFIG->{"FRITZWATCH_DISP_MODE"} );
  $iscalled ||= 2;
  my @methods = ({"index" => "0",
                  "method" => "Nummer",
                  "comment" => "Nur Nummer anzeigen"},
                 {"index" => "1",
                  "method" => "Nummer und Namen",
                  "comment" => "Nummer und Namen anzeigen"},
                 {"index" => "2",
                  "method" => "Namen",
                  "comment" => "Nur Namen anzeigen"},
                 {"index" => "3",
                  "method" => "Nichts",
                  "comment" => "Keine Anzeige"});
  
  foreach ( @methods ) {
    push ( @list, $_->{'method'},
           [ $_->{'comment'}, ($iscalled == $_->{'index'}) ? 1 : 0 ] );
  }
  
  $return = $d->radiolist(
    title => "Anzeige-Modus ( Angerufener )",
    text => "Namen werden nur angezeigt, wenn vorhanden. Ansonsten werden die Nummern angezeigt.",
    menuheight => 4,
    height => 13,
    list  => [ @list ],
  );
  
  if ( $return ) {
    foreach ( @methods ) {
      if ( $return eq $_->{'method'} ) {
        $SYSCONFIG->{"FRITZWATCH_DISP_MODE"} = $caller . ',' . $_->{'index'};
        last;
      }
    }
  }
}

sub MenuDisplay {
  my @list;
  my ( $caller, $iscalled ) = split ( /,/, $SYSCONFIG->{"FRITZWATCH_DISP_MODE"} );
  my @methods = ({"index" => "0",
                  "method" => "Nummer",
                  "comment" => "Nur Nummer anzeigen"},
                 {"index" => "1",
                  "method" => "Nummer und Namen",
                  "comment" => "Nummer und Namen anzeigen"},
                 {"index" => "2",
                  "method" => "Namen",
                  "comment" => "Nur Namen anzeigen"});
  
  foreach ( @methods ) {
    push ( @list, $_->{'method'},
           [ $_->{'comment'}, ($caller == $_->{'index'}) ? 1 : 0 ] );
  }
  
  $return = $d->radiolist(
    title => "Anzeige-Modus ( Anrufer )",
    text => "Namen werden nur angezeigt, wenn vorhanden. Ansonsten werden die Nummern angezeigt.",
    menuheight => 3,
    height => 12,
    list  => [ @list ],
  );
  
  if ( $return ) {
    foreach ( @methods ) {
      if ( $return eq $_->{'method'} ) {
        $SYSCONFIG->{"FRITZWATCH_DISP_MODE"} = $_->{'index'} . ',' . $iscalled;
        last;
      }
    }
  }
}

sub MenuNewCaller {
  my $exit = 0;
  my $number;
  while(not $exit) {
    if (not $number) {
      $return = $d->inputbox(
        title => "Nummer eingeben",
        entry => "",
        height => 9,
      );
      ($exit = 1) if($d->state ne "OK");
      if ($return =~ /^\*{0,2}\d+/) {
        if (exists $CALLER->{$return}) {
          $d->msgbox(
            text => ["Nummer \"$return\" existiert bereits."],
            height => 5,
          );
        }
        else {
          $number = $return;
        }
      }
      else {
        $d->msgbox(
          text => ["Bitte eine Nummer eingeben."],
          height => 5,
        );
      }
    }
    else {
      $return = $d->inputbox(
        title => "Namen eingeben",
        text => "Bitte einen Namen fuer Nummer \"$number\" eingeben.",
        entry => "",
        height => 9,
      );
      ($exit = 1) if($d->state ne "OK");
      if ($return =~ tr/\s//c) {
        $CALLER->{$number} = $return;
        $exit = 1;
      }
      else {
        $d->msgbox(
          text => ["Bitte einen Namen eingeben."],
          height => 5,
        );
      }
    }
  }
}

sub MenuFritzwatchOnOff {
  $return = $d->radiolist(
    title => "fritzwatch benutzen?",
    menuheight => 2,
    height => 9,
    list  => [ 
      "yes" , [ "" ,  ($SYSCONFIG->{"FRITZWATCH_ON"} eq "yes") ? 1 : 0 ],
      "no" , [ "" ,  ($SYSCONFIG->{"FRITZWATCH_ON"} eq "no") ? 1 : 0 ],
    ],
  );
  if($return){
    if ($return ne $SYSCONFIG->{"FRITZWATCH_ON"}) {
      $d->msgbox(
        text => "aenderung wird erst nach einem Neustart wirksam.",
        height => 5,
      );
      $SYSCONFIG->{"FRITZWATCH_ON"} = $return;
    }
  }
}

sub MenuMSNEdit {
  my $exit = 0;
  my @list;
  my %msn;
  my @new_msn;
  foreach (split(/,/,$SYSCONFIG->{"FRITZWATCH_MSN"})) {
    if ($_ =~ /^(\d+);(\w+)$/) {
      my($key, $value) = ($1, $2);
      $msn{$key} = $value;
    }
  }
  for my $key(sort(keys(%msn))) {
    if($key) {
      push(@list, $key);
      if ($msn{$key} eq "none") {
        push(@list, "");
      }
      else {
        push(@list, $msn{$key});
      }
    }
  }
  while(not $exit){
    $return = $d->menu(
      title => "MSN bearbeiten",
      text => "Bitte auswaehlen:",
      menuheight => 5,
      height => 12,
      list  => [ @list ],
    );
    ($exit = 1) if($d->state ne "OK");
    if ($return) {
      my $msn = $return;
      my $msn_name = ($msn{$msn} eq "none") ? "" : " $msn{$msn}";
      $return = $d->radiolist(
        title => "$msn$msn_name",
        menuheight => 2,
        height => 9,
        list  => [ 
          "Bearbeiten" , [ "" ,  1 ],
          "Loeschen" , [ "" ,  0 ],
        ],
      );
      if ($return eq "Bearbeiten") {
        $return = $d->radiolist(
          title => "$msn$msn_name",
          menuheight => 2,
          height => 9,
          list => [
            "MSN aendern" , [ "" , 0 ],
            "Namen aendern" , [ "" , 1 ],
          ],
        );
        if ($return eq "MSN aendern") {
          my $ok = 0;
          while(not $ok) {
            $return = $d->inputbox(
              title => "MSN eingeben",
              entry => $msn,
              height => 9,
            );
            ($ok = 1) if($d->state ne "OK");
            if ($return =~ /^\*{0,2}\d+/) {
              if (exists $msn{$return}) {
                $d->msgbox(
                  text => ["MSN \"$return\" existiert bereits.\nBitte erst loeschen."],
                  height => 6,
                );
                $ok = 1;
              }
              else {
                $msn{$return} = $msn{$msn};
                delete $msn{$msn};
                $ok = 1;
                $exit = 1;
              }
            }
            else {
              $d->msgbox(
                text => ["Bitte eine Nummer eingeben."],
                height => 5,
              );
            }
          }
        }
        if ($return eq "Namen aendern") {
          
          $return = $d->inputbox(
            title => "Namen eingeben",
            text => "Bitte einen Namen fuer MSN \"$msn\" eingeben.",
            entry => $msn{$msn},
            height => 9,
          );
          if ($return =~ tr/\s//c) {
            $msn{$msn} = $return;
          }
          else {
            $msn{$msn} = "none";
          }
          $exit = 1;
        }
      }
      if ($return eq "Loeschen") {
        $return = $d->yesno (
          title => "MSN loeschen",
          text => ["Soll der Eintrag \"$msn$msn_name\" wirklich geloescht werden?"],
          height => 8,
          defaultno => 1,
        );
        if ($return) {
          delete $msn{$msn};
          $exit = 1;
        }
      }
    }
  }
  foreach(sort(keys(%msn))) {
    push(@new_msn, $_ . ";" . $msn{$_}) if ($_);
  }
  $SYSCONFIG->{"FRITZWATCH_MSN"} = join(",",@new_msn);
}

sub MenuNewMSN {
  my $exit = 0;
  my %msn;
  my @new_msn;
  my $number;
  foreach (split(/,/,$SYSCONFIG->{"FRITZWATCH_MSN"})) {
    if ($_ =~ /^(\d+);(\w+)$/) {
      my($key, $value) = ($1, $2);
      $msn{$key} = $value;
    }
  }
  while(not $exit) {
    if (not $number) {
      $return = $d->inputbox(
        title => "MSN eingeben",
        entry => "",
        height => 9,
      );
      ($exit = 1) if($d->state ne "OK");
      if ($return =~ /^\*{0,2}\d+/) {
        if (exists $msn{$return}) {
          $d->msgbox(
            text => ["Nummer \"$return\" existiert bereits."],
            height => 5,
          );
        }
        else {
          $number = $return;
        }
      }
      else {
        $d->msgbox(
          text => ["Bitte eine Nummer eingeben."],
          height => 5,
        );
      }
    }
    else {
      $return = $d->inputbox(
        title => "Namen eingeben",
        text => "Bitte einen Namen fuer MSN \"$number\" eingeben.",
        entry => "",
        height => 9,
      );
      ($exit = 1) if($d->state ne "OK");
      if ($return =~ tr/\s//c) {
        $msn{$number} = $return;
      }
      else {
        $msn{$number} = "none";
      }
      $exit = 1;
    }
  }
  foreach(sort(keys(%msn))) {
    push(@new_msn, $_ . ";" . $msn{$_}) if ($_);
  }
  $SYSCONFIG->{"FRITZWATCH_MSN"} = join(",",@new_msn);
}

sub MenuCallerEdit {
  my $exit = 0;
  my @list;
  for my $key(sort( { $a cmp $b } keys(%$CALLER) )) {
    if($key and $key ne 'path2config') {
      push(@list, $key);
      push(@list, $CALLER->{$key});
    }
  }
  while(!$exit) {
    my $return = $d->menu(
      title => "Nummern bearbeiten",
      text  => "Bitte auswaehlen:",
      menuheight => 10,
      height => 17,
      list  => [ @list ],
    );
    ($exit = 1) if($d->state ne "OK");
    if ($return) {
      my $number = $return;
      $return = $d->radiolist(
        title => "$number $CALLER->{$number}",
        menuheight => 2,
        height => 9,
        list  => [ 
          "Bearbeiten" , [ "" ,  1 ],
          "Loeschen" , [ "" ,  0 ],
        ],
      );
      if ($return eq "Bearbeiten") {
        $return = $d->radiolist(
          title => "$number $CALLER->{$number}",
          menuheight => 2,
          height => 9,
          list => [
            "Nummer aendern" , [ "" , 0 ],
            "Namen aendern" , [ "" , 1 ],
          ],
        );
        if ($return eq "Nummer aendern") {
          my $ok = 0;
          while(not $ok) {
            $return = $d->inputbox(
              title => "Nummer eingeben",
              entry => $number,
              height => 9,
            );
            ($ok = 1) if($d->state ne "OK");
            if ($return =~ /^\*{0,2}\d+/) {
              if (exists $CALLER->{$return}) {
                $d->msgbox(
                  text => ["Nummer \"$return\" existiert bereits.\nBitte erst loeschen."],
                  height => 6,
                );
                $ok = 1;
              }
              else {
                $CALLER->{$return} = $CALLER->{$number};
                delete $CALLER->{$number};
                $ok = 1;
                $exit = 1;
              }
            }
            else {
              $d->msgbox(
                text => ["Bitte eine Nummer eingeben."],
                height => 5,
              );
            }
          }
        }
        if ($return eq "Namen aendern") {
          $return = $d->inputbox(
            title => "Namen eingeben",
            text => "Bitte einen Namen fuer Nummer \"$number\" eingeben.",
            entry => $CALLER->{$number},
            height => 9,
          );
          $CALLER->{$number} = $return;
          $exit = 1;
        }
      }
      if ($return eq "Loeschen") {
        $return = $d->yesno (
          title => "Nummer loeschen",
          text => ["Soll der Eintrag \"$number $CALLER->{$number}\" wirklich geloescht werden?"],
          height => 8,
          defaultno => 1,
        );
        if ($return) {
          delete $CALLER->{$number};
          $exit = 1;
        }
      }
    }
  }
}

sub notRootWarning {
  if($< != 0) {
    $d->msgbox(
      text       => ["Setup muss als Benutzer root gestartet werden."],
      width      => 50,
      height     => 6,
      "ok-label" => "Beenden"
    );
    exit(1);
  }
}
