#!/usr/bin/perl

###
# Dieses Skript ist die Perl Variante des Skriptes
# fritzwatch.sh von helmut (at) helmutauer (dot) de
# 
# Bugs bitte melden an neves-enin (at) gmx (dot) net
###

use IO::Socket;
use IO::Socket::INET 1.26;
use strict;
use Getopt::Std;
use VDRTools::System;
use VDRTools::Config;
use VDRTools::Sysconfig;
use VDRTools::File;
use VDRTools::Network qw(vt_svdrpsend);

my $CALL_DB       = "/root/caller.db";

my $VERSION       = "0.0.5";

my %args;

######################################################################

getopts("v", \%args);

if ($args{'v'}){
	print( $0 . ' ' . $VERSION . "\n" );
	exit (0);
}

my $oldpid = vt_pidof( vt_filename( $0 ) );
if ( $oldpid and $oldpid != $$ ) {
	exit(0);
}

vt_fork_to_bg();

while(1) {
	my $sysconfig = VDRTools::Sysconfig->new();
	my $socket = new IO::Socket::INET (
		PeerAddr => $sysconfig->{FRITZWATCH_FRITZBOX} || 'fritz.box',
		PeerPort => 1012,
		Proto => 'tcp'
	)|| die;
	
	while(<$socket>) {
		my @details = split(/;/,$_);
		my $msn = {};
		my $caller_db = VDRTools::Config->new( $CALL_DB );
		$sysconfig = VDRTools::Sysconfig->new();
		if ( defined( $sysconfig->{FRITZWATCH_MSN} ) ) {
			foreach( split( /,/, $sysconfig->{FRITZWATCH_MSN} ) ) {
				my ( $num, $name ) = split(/;/,$_);
				$msn->{$num} = defined( $name ) ? ( ( $name eq 'none' ) ? '' : $name ) : '';
			}
		}
		if ( ( $details[1] eq "RING" ) and defined( $msn->{$details[4]} ) ) {
			my $number = $details[3];
			my $iscalled = $details[4];
			my $caller = '';
			my ( $disp_c, $disp_is_c ) = split( /,/, $sysconfig->{FRITZWATCH_DISP_MODE} || '2,2' );
			my $msg = 'Anruf von';
			if($number) {
				unless ( exists( $caller_db->{$number} ) ) {
					$caller_db->{$number} = '';
					$caller_db->save();
				}
				$msg .= ' ' . $caller_db->{$number} if ( ( $disp_c == 1 or $disp_c == 2 ) and $caller_db->{$number} );
				$msg .= ' ' . $number if ( ( $disp_c == 0 or $disp_c == 0 ) or ( $disp_c == 2 and not $caller_db->{$number} ) );
			}
			else {
				$msg .= ' unbekanntem Teilnehmer';
			}
			$msg .= ' fr' if ( $disp_is_c != 3 );
			$msg .= ' ' . $msn->{$iscalled} if ( ( $disp_is_c == 1 or $disp_is_c == 2 ) and $msn->{$iscalled} );
			$msg .= ' ' . $iscalled if ( ( $disp_is_c == 0 or $disp_is_c == 1 ) or ( $disp_is_c == 2 and not $msn->{$iscalled} ) );
			$msg .= '!';
			vt_svdrpsend( 'MESG ' . $msg );
			system(sprintf("logger \"Caller: %s\"", $msg));
			sleep(2);
		}
	}
	$socket->close();
	sleep(10);
}
exit(0);
